package hu.bme.mit.books.model.tests;

import static org.junit.Assert.*;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import hu.bme.mit.books.model.Book;

import org.junit.After;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

public class BookListenerTest {

	private final class CounterListener implements
			PropertyChangeListener {
		int count = 0;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			assertEquals("pagenum", evt.getPropertyName());
			count++;
		}
	}

	private Book book;

	@Before
	public void setUp() throws Exception {
		book = new Book();
		book.setAuthor("Test author");
		book.setTitle("Test title");
		book.setPagenum(123);
		book.setRead(true);
	}

	@After
	public void tearDown() throws Exception {
		book = null;
	}

	@Test
	public void testAddTitlePropertyChangeListener() {
		addListener("title");
		book.setTitle("Test");
	}
	
	@Test
	public void testAddPageNumPropertyChangeListener() {
		addListener("pagenum");
		book.setPagenum(1234);
	}

	private void addListener(final String property) {
		book.addPropertyChangeListener(property, new PropertyChangeListener() {
			
			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				assertEquals(property, evt.getPropertyName());	
			}
		});
	}

	@Test
	public void testAddDuplicatePageNumPropertyChangeListener() {
		CounterListener listener = new CounterListener();
		book.addPropertyChangeListener("pagenum", listener);
		book.addPropertyChangeListener("pagenum", listener);
		book.setPagenum(1234);
		assertEquals(2, listener.count);
	}
	
	@Test
	public void testAddMultipleListeners() {
		addListener("title");
		addListener("pagenum");
		book.setTitle("Test");
		book.setPagenum(1024);
	}
	@Test
	@Ignore
	public void testRemovePropertyChangeListener() {
		fail("Not yet implemented");
	}

}
